@@ -23,9 +23,7 @@
 #define GNUMERIC_HTML_BOOT_H
 
 #include "gnumeric.h"
-#include "gnumeric-util.h"
-#include "../../src/plugin.h"
-
-PluginInitResult init_plugin (CommandContext *context, PluginData *pd);
+#include "plugin.h"
+#include "plugin-util.h"
 
 #endif
