@@ -1,12 +1,16 @@
 #ifndef GNUMERIC_ANALYSIS_TOOLS_H
 #define GNUMERIC_ANALYSIS_TOOLS_H
 
-#include "numbers.h"
 #include "gnumeric.h"
+#include "numbers.h"
+
+typedef enum {
+	GROUPED_BY_ROW = 0,
+	GROUPED_BY_COL = 1,
+	GROUPED_BY_AREA = 2,
+	GROUPED_BY_BIN = 3
+} group_by_t;
 
-typedef struct {
-        GtkWidget *entry[5];
-} stat_tool_t;
 
 typedef struct {
         gboolean summary_statistics;
@@ -24,8 +28,7 @@ typedef enum {
 } random_distribution_t;
 
 typedef struct {
-        int start_col, start_row;
-        int end_col, end_row;
+	Value *range;
 } discrete_random_tool_t;
 
 typedef struct {
@@ -89,8 +92,17 @@ typedef struct {
         int                         start_col, cols;
         int                         start_row, rows;
         gboolean                    labels_flag;
+	gboolean                    autofit_flag;
 } data_analysis_output_t;
 
+typedef struct {
+	gboolean max_given;
+	gboolean min_given;
+	gnum_float max;
+	gnum_float min;
+	gint n;
+} histogram_calc_bin_info_t;
+
 
 void autofit_column  (data_analysis_output_t *dao, int col);
 void set_cell        (data_analysis_output_t *dao, int col, int row, const char *text);
