@@ -3,11 +3,13 @@
  * cell.c: Cell content and simple management.
  *
  * Author:
- *    Jody Goldberg 2000, 2001 (jgoldberg@home.com)
+ *    Jody Goldberg 2000, 2001 (jody@gnome.org)
  *    Miguel de Icaza 1998, 1999 (miguel@kernel.org)
  */
-#include "config.h"
+#include <gnumeric-config.h>
+#include "gnumeric.h"
 #include "cell.h"
+
 #include "workbook.h"
 #include "sheet.h"
 #include "expr.h"
@@ -19,8 +21,6 @@
 #include "eval.h"
 #include "sheet-style.h"
 
-extern int dependency_debugging;
-
 /**
  * cell_dirty : Mark the sheet containing the cell as being dirty.
  * @cell : the dirty cell.
@@ -65,6 +65,7 @@ cell_cleanout (Cell *cell)
 			dependent_unlink (CELL_TO_DEP (cell), &cell->pos);
 		expr_tree_unref (cell->base.expression);
 		cell->base.expression = NULL;
+		cell->base.flags &= ~(CELL_HAS_EXPRESSION);
 	}
 
 	if (cell->value) {
@@ -80,7 +81,6 @@ cell_cleanout (Cell *cell)
 		cell->format = NULL;
 	}
 
-	cell->base.flags &= ~(CELL_HAS_EXPRESSION|DEPENDENT_IN_RECALC_QUEUE|DEPENDENT_NEEDS_RECALC);
 	cell_dirty (cell);
 }
 
@@ -106,7 +106,7 @@ cell_copy (Cell const *cell)
 
 	/* The new cell is not linked into any of the major management structures */
 	new_cell->base.sheet = NULL;
-	new_cell->base.flags &= ~(DEPENDENT_IN_RECALC_QUEUE|DEPENDENT_NEEDS_RECALC|CELL_IN_SHEET_LIST|DEPENDENT_IN_EXPR_LIST);
+	new_cell->base.flags &= ~(DEPENDENT_NEEDS_RECALC|CELL_IN_SHEET_LIST|DEPENDENT_IN_EXPR_LIST);
 
 	/* now copy properly the rest */
 	if (cell_has_expr (new_cell))
@@ -210,7 +210,7 @@ cell_eval_content (Cell *cell)
 	max_iteration = cell->base.sheet->workbook->iteration.max_number;
 
 iterate :
-	v = eval_expr (&pos, cell->base.expression, EVAL_STRICT);
+	v = expr_eval (cell->base.expression, &pos, EVAL_STRICT);
 	if (v == NULL)
 		v = value_new_error (&pos, "Internal error");
 
@@ -253,7 +253,15 @@ iterate :
 		if (cell->value != NULL)
 			value_release (cell->value);
 		cell->value = v;
-		cell_render_value (cell, TRUE);
+
+		/* Optimization : Since we don't span calculated cells
+		 * it is ok, to wipe rendered values.  The drawing routine
+		 * will handle it.
+		 */
+		if (cell->rendered_value != NULL) {
+			rendered_value_destroy (cell->rendered_value);
+			cell->rendered_value = NULL;
+		}
 	}
 
 	if (iterating == cell)
@@ -263,7 +271,6 @@ iterate :
 	printf ("} (%d)\n", iterating == NULL);
 #endif
 	cell->base.flags &= ~DEPENDENT_BEING_CALCULATED;
-	sheet_redraw_cell (cell);
 	return iterating == NULL;
 }
 
@@ -327,24 +334,18 @@ void
 cell_set_text (Cell *cell, char const *text)
 {
 	StyleFormat *format;
-	Value *val;
-	ExprTree *expr;
-	EvalPos pos;
-	MStyle *mstyle;
-	StyleFormat *cformat;
+	ExprTree    *expr;
+	Value	    *val;
+	ParsePos     pos;
 
 	g_return_if_fail (cell != NULL);
 	g_return_if_fail (text != NULL);
 	g_return_if_fail (!cell_is_partial_array (cell));
 
-	mstyle = cell_get_mstyle (cell);
-	cformat = mstyle_get_format (mstyle);
-	format = parse_text_value_or_expr (eval_pos_init_cell (&pos, cell),
-					   text, &val, &expr, cformat);
+	format = parse_text_value_or_expr (parse_pos_init_cell (&pos, cell),
+		text, &val, &expr, mstyle_get_format (cell_get_mstyle (cell)));
 
 	if (val != NULL) {	/* String was a value */
-		if (format)
-			style_format_ref (format);
 		cell_cleanout (cell);
 		cell->value = val;
 		/* parse_text already refed the format */
@@ -353,6 +354,7 @@ cell_set_text (Cell *cell, char const *text)
 	} else {		/* String was an expression */
 		cell_set_expr (cell, expr, format);
 		expr_tree_unref (expr);
+		style_format_unref (format);
 	}
 }
 
@@ -402,11 +404,6 @@ cell_assign_value (Cell *cell, Value *v, StyleFormat *opt_fmt)
  * WARNING : This is an internal routine that does not queue redraws,
  *           does not auto-resize, and does not calculate spans.
  *
- * FIXME FIXME FIXME : The current format code only checks against the list of
- * canned formats.  Therefore the rendered string MAY NOT BE PARSEABLE!! if the
- * user has assigned a non-std format.  We need to improve the parser to handle
- * all formats that exist within the workbook.
- *
  * NOTE : This DOES check for array partitioning.
  */
 void
@@ -572,7 +569,7 @@ cell_set_array_formula (Sheet *sheet,
 	g_return_if_fail (col_a <= col_b);
 	g_return_if_fail (row_a <= row_b);
 
-	wrapper = expr_tree_new_array (0, 0, num_rows, num_cols);
+	wrapper = expr_tree_new_array (0, 0, num_cols, num_rows);
 	wrapper->array.corner.value = NULL;
 	wrapper->array.corner.expr = formula;
 	cell_set_expr_internal (corner, wrapper, NULL);
@@ -586,7 +583,7 @@ cell_set_array_formula (Sheet *sheet,
 				continue;
 
 			cell = sheet_cell_fetch (sheet, col_a + x, row_a + y);
-			wrapper = expr_tree_new_array (x, y, num_rows, num_cols);
+			wrapper = expr_tree_new_array (x, y, num_cols, num_rows);
 			cell_set_expr_internal (cell, wrapper, NULL);
 			dependent_link (CELL_TO_DEP (cell), &cell->pos);
 			expr_tree_unref (wrapper);
@@ -666,7 +663,7 @@ cell_is_partial_array (Cell const *cell)
  * @dynamic_width : Allow format to depend on column width.
  *
  * TODO :
- * The reason the rendered values are stored seperately from the Cell is
+ * The reason the rendered values are stored separately from the Cell is
  * that in the future only visible cells will be rendered.  The render
  * will be SheetControl specific to allow for multiple zooms and different
  * display resolutions.
@@ -689,7 +686,6 @@ cell_render_value (Cell *cell, gboolean dynamic_width)
 	rendered_value_calc_size_ext (cell, mstyle);
 }
 
-
 MStyle *
 cell_get_mstyle (Cell const *cell)
 {
@@ -801,23 +797,21 @@ cellpos_cmp (CellPos const * a, CellPos const * b)
 	return (a->row == b->row && a->col == b->col);
 }
 
-
-
 CellComment *
 cell_has_comment_pos (const Sheet *sheet, const CellPos *pos)
 {
 	Range r;
-	GList *comments;
+	GSList *comments;
 	CellComment *res;
 
 	r.start = r.end = *pos;
-	comments = sheet_get_objects (sheet, &r, CELL_COMMENT_TYPE);
+	comments = sheet_objects_get (sheet, &r, CELL_COMMENT_TYPE);
 	if (!comments)
 		return NULL;
 
 	/* This assumes just one comment per cell.  */
 	res = comments->data;
-	g_list_free (comments);
+	g_slist_free (comments);
 	return res;
 }
 
