@@ -5,14 +5,14 @@
  *  Miguel de Icaza (miguel@gnu.org)
  *
  */
-#include <config.h>
-#include <glib.h>
-#include <libgnome/gnome-defs.h>
-#include <libgnome/gnome-i18n.h>
-#include "gnumeric.h"
+#include <gnumeric-config.h>
+#include <gnumeric.h>
 #include "dialogs.h"
-#include "workbook.h"
-#include "gui-util.h"
+
+#include <workbook.h>
+#include <gui-util.h>
+
+#include <libgnome/gnome-i18n.h>
 
 #define ABOUT_KEY          "about-dialog"
 
@@ -79,7 +79,7 @@ dialog_about (WorkbookControlGUI *wbcg)
 		return;
 
         about = gnome_about_new (_("Gnumeric"), VERSION,
-				 _("(C) 1998-2001 Miguel de Icaza"),
+				 _("(C) 1998-2000 Miguel de Icaza, 2001-2002 Jody Goldberg"),
 				 authors,
 				 NULL,
 				 "gnome-gnumeric.png");
