@@ -11,31 +11,36 @@
 #define DEBUG_GOAL_SEEK
 #endif
 
-#include <config.h>
-#include <gnome.h>
+#include <gnumeric-config.h>
+#include "numbers.h"
+#include "gnumeric.h"
+#include "goal-seek.h"
+
+#include "mathfunc.h"
 #include <stdlib.h>
 #include <math.h>
-#include "goal-seek.h"
-#include "gnumeric.h"
-#include "mathfunc.h"
-
-/*
- * This value should be comfortably within the relative precision of a float_t.
- * For doubles, that means something like 10^-DBL_DIG.
- */
-#define XRELSTEP_MIN 1e-12
+#include <limits.h>
 
 #ifdef DEBUG_GOAL_SEEK
 #include <stdio.h>
 #endif
 
 static gboolean
-update_data (float_t x, float_t y, GoalSeekData *data)
+update_data (gnum_float x, gnum_float y, GoalSeekData *data)
 {
 	if (y > 0) {
 		if (data->havexpos) {
-			if (data->havexneg &&
-			    fabs (x - data->xneg) < fabs (data->xpos - data->xneg)) {
+			if (data->havexneg) {
+				/*
+				 * When we have pos and neg, prefer the new point only
+				 * if it makes the pos-neg x-internal smaller.
+				 */
+				if (fabs (x - data->xneg) < fabs (data->xpos - data->xneg)) {
+					data->xpos = x;
+					data->ypos = y;
+				}
+			} else if (y < data->ypos) {
+				/* We have pos only and our neg y is closer to zero.  */
 				data->xpos = x;
 				data->ypos = y;
 			}
@@ -47,11 +52,21 @@ update_data (float_t x, float_t y, GoalSeekData *data)
 		return FALSE;
 	} else if (y < 0) {
 		if (data->havexneg) {
-			if (data->havexpos &&
-			    fabs (x - data->xpos) < fabs (data->xpos - data->xneg)) {
+			if (data->havexpos) {
+				/*
+				 * When we have pos and neg, prefer the new point only
+				 * if it makes the pos-neg x-internal smaller.
+				 */
+				if (fabs (x - data->xpos) < fabs (data->xpos - data->xneg)) {
+					data->xneg = x;
+					data->yneg = y;
+				}
+			} else if (-y < -data->yneg) {
+				/* We have neg only and our neg y is closer to zero.  */
 				data->xneg = x;
 				data->yneg = y;
 			}
+
 		} else {
 			data->xneg = x;
 			data->yneg = y;
@@ -71,14 +86,12 @@ update_data (float_t x, float_t y, GoalSeekData *data)
  * in a single point.
  */
 static GoalSeekStatus
-fake_df (GoalSeekFunction f, float_t x, float_t *dfx, float_t xrelstep,
+fake_df (GoalSeekFunction f, gnum_float x, gnum_float *dfx, gnum_float xstep,
 	 GoalSeekData *data, void *user_data)
 {
-	float_t xstep, xl, xr, yl, yr;
+	gnum_float xl, xr, yl, yr;
 	GoalSeekStatus status;
 
-	xstep = fabs (x) * xrelstep;
-
 	xl = x - xstep;
 	if (xl < data->xmin)
 		xl = x;
@@ -87,19 +100,31 @@ fake_df (GoalSeekFunction f, float_t x, float_t *dfx, float_t xrelstep,
 	if (xr > data->xmax)
 		xr = x;
 
-	if (xl == xr)
+	if (xl == xr) {
+#ifdef DEBUG_GOAL_SEEK
+		printf ("==> xl == xr\n");
+#endif
 		return GOAL_SEEK_ERROR;
+	}
 
 	status = f (xl, &yl, user_data);
-	if (status != GOAL_SEEK_OK)
+	if (status != GOAL_SEEK_OK) {
+#ifdef DEBUG_GOAL_SEEK
+		printf ("==> failure at xl\n");
+#endif
 		return status;
+	}
 
 	status = f (xr, &yr, user_data);
-	if (status != GOAL_SEEK_OK)
+	if (status != GOAL_SEEK_OK) {
+#ifdef DEBUG_GOAL_SEEK
+		printf ("==> failure at xr\n");
+#endif
 		return status;
+	}
 
 	*dfx = (yr - yl) / (xr - xl);
-	return GOAL_SEEK_OK;
+	return finite (*dfx) ? GOAL_SEEK_OK : GOAL_SEEK_ERROR;
 }
 
 void
@@ -117,10 +142,10 @@ goal_seek_initialise (GoalSeekData *data)
  */
 GoalSeekStatus
 goal_seek_point (GoalSeekFunction f, GoalSeekData *data,
-		 void *user_data, float_t x0)
+		 void *user_data, gnum_float x0)
 {
 	GoalSeekStatus status;
-	float_t y0;
+	gnum_float y0;
 
 	if (x0 < data->xmin || x0 > data->xmax)
 		return GOAL_SEEK_ERROR;
@@ -150,14 +175,17 @@ goal_seek_point (GoalSeekFunction f, GoalSeekData *data,
  */
 GoalSeekStatus
 goal_seek_newton (GoalSeekFunction f, GoalSeekFunction df,
-		  GoalSeekData *data, void *user_data, float_t x0)
+		  GoalSeekData *data, void *user_data, gnum_float x0)
 {
 	int iterations;
-	float_t xrelstep = 1e-3;
-	float_t precision = data->precision / 2;
+	gnum_float precision = data->precision / 2;
+
+#ifdef DEBUG_GOAL_SEEK
+	printf ("goal_seek_newton\n");
+#endif
 
 	for (iterations = 0; iterations < 20; iterations++) {
-		float_t x1, y0, df0, stepsize;
+		gnum_float x1, y0, df0, stepsize;
 		GoalSeekStatus status;
 
 		/* Check whether we have left the valid interval.  */
@@ -168,13 +196,29 @@ goal_seek_newton (GoalSeekFunction f, GoalSeekFunction df,
 		if (status != GOAL_SEEK_OK)
 			return status;
 
+#ifdef DEBUG_GOAL_SEEK
+		printf ("x0 = %.20g\n", x0);
+		printf ("                                        y0 = %.20g\n", y0);
+#endif
+
 		if (update_data (x0, y0, data))
 			return GOAL_SEEK_OK;
 
 		if (df)
 			status = df (x0, &df0, user_data);
-		else
-			status = fake_df (f, x0, &df0, xrelstep, data, user_data);
+		else {
+			gnum_float xstep;
+
+			if (fabs (x0) < 1e-10)
+				if (data->havexneg && data->havexpos)
+					xstep = fabs (data->xpos - data->xneg) / 1e6;
+				else
+					xstep = (data->xmax - data->xmin) / 1e6;
+			else
+				xstep = fabs (x0) / 1e6;
+
+			status = fake_df (f, x0, &df0, xstep, data, user_data);
+		}
 		if (status != GOAL_SEEK_OK)
 			return status;
 
@@ -182,7 +226,11 @@ goal_seek_newton (GoalSeekFunction f, GoalSeekFunction df,
 		if (df0 == 0)
 			return GOAL_SEEK_ERROR;
 
-		x1 = x0 - y0 / df0;
+		/*
+		 * Overshoot slightly to prevent us from staying on
+		 * just one side of the root.
+		 */
+		x1 = x0 - 1.000001 * y0 / df0;
 		if (x1 == x0) {
 			data->root = x0;
 			return GOAL_SEEK_OK;
@@ -191,21 +239,16 @@ goal_seek_newton (GoalSeekFunction f, GoalSeekFunction df,
 		stepsize = fabs (x1 - x0) / (fabs (x0) + fabs (x1));
 
 #ifdef DEBUG_GOAL_SEEK
-		printf ("x0 = %.20g\n", x0);
-		printf ("                                        y0 = %.20g\n", y0);
+		printf ("                                        df0 = %.20g\n", df0);
 		printf ("                                        ss = %.20g\n", stepsize);
 #endif
 
+		x0 = x1;
+
 		if (stepsize < precision) {
 			data->root = x0;
 			return GOAL_SEEK_OK;
 		}
-
-		/* As we get closer to the root, improve the derivation.  */
-		if (stepsize * 1000 < xrelstep)
-			xrelstep = MAX (xrelstep / 1000, XRELSTEP_MIN);
-
-		x0 = x1;
 	}
 
 	return GOAL_SEEK_ERROR;
@@ -227,33 +270,53 @@ goal_seek_newton (GoalSeekFunction f, GoalSeekFunction df,
  * the secant method).
  */
 
-#define SECANT_P(i) ((i) % 8 == 6)
-#define RIDDER_P(i) ((i) % 8 < 6)
-
 GoalSeekStatus
 goal_seek_bisection (GoalSeekFunction f, GoalSeekData *data, void *user_data)
 {
 	int iterations;
+	gnum_float stepsize;
+	int newton_submethod = 0;
+
+#ifdef DEBUG_GOAL_SEEK
+	printf ("goal_seek_bisection\n");
+#endif
 
 	if (!data->havexpos || !data->havexneg)
 		return GOAL_SEEK_ERROR;
 
-	for (iterations = 0; iterations < 60; iterations++) {
-		float_t xmid, ymid, stepsize;
-		GoalSeekStatus status;
+	stepsize = fabs (data->xpos - data->xneg)
+		/ (fabs (data->xpos) + fabs (data->xneg));
 
-		if (SECANT_P (iterations)) {
-			/* Use secant method.  */
+	/* log_2 (10) = 3.3219 < 4.  */
+	for (iterations = 0; iterations < 100 + DBL_DIG * 4; iterations++) {
+		gnum_float xmid, ymid;
+		GoalSeekStatus status;
+		enum { M_SECANT, M_RIDDER, M_NEWTON, M_MIDPOINT } method;
+
+		method = (iterations % 4 == 0)
+			? M_RIDDER
+			: ((iterations % 4 == 2)
+			   ? M_NEWTON
+			   : M_MIDPOINT);
+
+	again:
+		switch (method) {
+		default:
+			abort ();
+
+		case M_SECANT:
 			xmid = data->xpos - data->ypos *
 				((data->xneg - data->xpos) /
 				 (data->yneg - data->ypos));
-		} else if (RIDDER_P (iterations)) {
-			float_t det;
+			break;
+
+		case M_RIDDER: {
+			gnum_float det;
 
 			xmid = (data->xpos + data->xneg) / 2;
 			status = f (xmid, &ymid, user_data);
 			if (status != GOAL_SEEK_OK)
-				return status;
+				continue;
 			if (ymid == 0) {
 				update_data (xmid, ymid, data);
 				return GOAL_SEEK_OK;
@@ -261,34 +324,97 @@ goal_seek_bisection (GoalSeekFunction f, GoalSeekData *data, void *user_data)
 
 			det = sqrt (ymid * ymid - data->ypos * data->yneg);
 			if (det == 0)
-				return GOAL_SEEK_ERROR;
+				 /* This might happen with underflow, I guess. */
+				continue;
 
 			xmid += (xmid - data->xpos) * ymid / det;
-		} else {
-			/* Use plain midpoint.  */
+			break;
+		}
+
+		case M_MIDPOINT:
 			xmid = (data->xpos + data->xneg) / 2;
+			break;
+
+		case M_NEWTON: {
+			gnum_float x0, y0, xstep, df0;
+
+			/* This method is only effective close-in.  */
+			if (stepsize > 0.1) {
+				method = M_MIDPOINT;
+				goto again;
+			}
+
+			switch (newton_submethod++ % 4) {
+			case 0:	x0 = data->xpos; x0 = data->ypos; break;
+			case 2: x0 = data->xneg; y0 = data->yneg; break;
+			default:
+			case 3:
+			case 1:
+				x0 = (data->xpos + data->xneg) / 2;
+
+				status = f (x0, &y0, user_data);
+				if (status != GOAL_SEEK_OK)
+					continue;
+			}
+
+			xstep = fabs (data->xpos - data->xneg) / 1e6;
+			status = fake_df (f, x0, &df0, xstep, data, user_data);
+			if (status != GOAL_SEEK_OK)
+				continue;
+
+			if (df0 == 0)
+				continue;
+
+			/*
+			 * Overshoot by 1% to prevent us from staying on
+			 * just one side of the root.
+			 */
+			xmid = x0 - 1.01 * y0 / df0;
+			if ((xmid < data->xpos && xmid < data->xneg) ||
+			    (xmid > data->xpos && xmid > data->xneg))
+				/* We left the interval.  */
+				continue;
+		}
 		}
 
 		status = f (xmid, &ymid, user_data);
 		if (status != GOAL_SEEK_OK)
-			return status;
+			continue;
 
-		if (update_data (xmid, ymid, data))
+#ifdef DEBUG_GOAL_SEEK
+		{
+			const char *themethod;
+			switch (method) {
+			case M_MIDPOINT: themethod = "midpoint"; break;
+			case M_RIDDER: themethod = "Ridder"; break;
+			case M_SECANT: themethod = "secant"; break;
+			case M_NEWTON: themethod = "Newton"; break;
+			default: themethod = "?";
+			}
+
+			printf ("xmid = %.20g (%s)\n", xmid, themethod);
+			printf ("                                        ymid = %.20g\n", ymid);
+		}
+#endif
+
+		if (update_data (xmid, ymid, data)) {
 			return GOAL_SEEK_OK;
+		}
 
-		stepsize = fabs (data->xpos - data->xneg)\
+		stepsize = fabs (data->xpos - data->xneg)
 			/ (fabs (data->xpos) + fabs (data->xneg));
 
 #ifdef DEBUG_GOAL_SEEK
-		printf ("xmid = %.20g (%s)\n", xmid,
-			SECANT_P (iterations) ? "secant" :
-			(RIDDER_P (iterations) ? "Ridder" :
-			 "mid-point"));
-		printf ("                                        ymid = %.20g\n", ymid);
 		printf ("                                          ss = %.20g\n", stepsize);
 #endif
 
 		if (stepsize < data->precision) {
+			if (data->yneg < ymid)
+				ymid = data->yneg, xmid = data->xneg;
+
+			if (data->ypos < ymid)
+				ymid = data->ypos, xmid = data->xpos;
+
 			data->root = xmid;
 			return GOAL_SEEK_OK;
 		}
@@ -302,7 +428,7 @@ goal_seek_bisection (GoalSeekFunction f, GoalSeekData *data, void *user_data)
 GoalSeekStatus
 goal_seek_trawl_uniformly (GoalSeekFunction f,
 			   GoalSeekData *data, void *user_data,
-			   float_t xmin, float_t xmax,
+			   gnum_float xmin, gnum_float xmax,
 			   int points)
 {
 	int i;
@@ -311,7 +437,7 @@ goal_seek_trawl_uniformly (GoalSeekFunction f,
 		return GOAL_SEEK_ERROR;
 
 	for (i = 0; i < points; i++) {
-		float_t x, y;
+		gnum_float x, y;
 		GoalSeekStatus status;
 
 		if (data->havexpos && data->havexneg)
@@ -340,7 +466,7 @@ goal_seek_trawl_uniformly (GoalSeekFunction f,
 GoalSeekStatus
 goal_seek_trawl_normally (GoalSeekFunction f,
 			  GoalSeekData *data, void *user_data,
-			  float_t mu, float_t sigma,
+			  gnum_float mu, gnum_float sigma,
 			  int points)
 {
 	int i;
@@ -349,7 +475,7 @@ goal_seek_trawl_normally (GoalSeekFunction f,
 		return GOAL_SEEK_ERROR;
 
 	for (i = 0; i < points; i++) {
-		float_t x, y;
+		gnum_float x, y;
 		GoalSeekStatus status;
 
 		if (data->havexpos && data->havexneg)
@@ -380,14 +506,14 @@ goal_seek_trawl_normally (GoalSeekFunction f,
 
 #ifdef STANDALONE
 static GoalSeekStatus
-f (float_t x, float_t *y, void *user_data)
+f (gnum_float x, gnum_float *y, void *user_data)
 {
 	*y = x * x - 2;
 	return GOAL_SEEK_OK;
 }
 
 static GoalSeekStatus
-df (float_t x, float_t *y, void *user_data)
+df (gnum_float x, gnum_float *y, void *user_data)
 {
 	*y = 2 * x;
 	return GOAL_SEEK_OK;
